==> create-fly-1.21.11-pre1-6.0.8-1-sources.jar: com/zurrtum/create/content/logistics/funnel/AbstractFunnelBlock.java <==
package com.zurrtum.create.content.logistics.funnel;

import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.content.equipment.wrench.IWrenchable;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import com.zurrtum.create.foundation.blockEntity.behaviour.BlockEntityBehaviour;
import com.zurrtum.create.foundation.blockEntity.behaviour.filtering.ServerFilteringBehaviour;
import com.zurrtum.create.foundation.blockEntity.behaviour.inventory.InvManipulationBehaviour;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2586;
import net.minecraft.class_2591;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3610;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9904;
import org.jetbrains.annotations.Nullable;

public abstract class AbstractFunnelBlock extends class_2248 implements IBE<FunnelBlockEntity>, IWrenchable, ProperWaterloggedBlock, NeighborUpdateListeningBlock {

    public static final class_2746 POWERED = class_2741.field_12484;

    protected AbstractFunnelBlock(class_2251 p_i48377_1_) {
        super(p_i48377_1_);
        method_9590(method_9564().method_11657(POWERED, false).method_11657(WATERLOGGED, false));
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        return withWater(method_9564().method_11657(POWERED, context.method_8045().method_49803(context.method_8037())), context);
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pDirection,
        class_2338 pNeighborPos,
        class_2680 pNeighborState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        super.method_9515(builder.method_11667(POWERED, WATERLOGGED));
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 level, class_2338 pos, class_2248 sourceBlock, class_2338 fromPos, boolean isMoving) {
        if (level.method_8608())
            return;
        InvManipulationBehaviour behaviour = BlockEntityBehaviour.get(level, pos, InvManipulationBehaviour.TYPE);
        if (behaviour != null)
            behaviour.onNeighborChanged(fromPos);
    }

    @Override
    public void method_9612(class_2680 state, class_1937 level, class_2338 pos, class_2248 block, @Nullable class_9904 wireOrientation, boolean isMoving) {
        if (level.method_8608())
            return;
        if (!level.method_8397().method_8677(pos, this))
            level.method_64310(pos, this, 1);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 worldIn, class_2338 pos, class_5819 r) {
        boolean previouslyPowered = state.method_11654(POWERED);
        if (previouslyPowered != worldIn.method_49803(pos))
            worldIn.method_8652(pos, state.method_28493(POWERED), class_2248.field_31028);
    }

    public static class_1799 tryInsert(class_1937 worldIn, class_2338 pos, class_1799 toInsert, boolean simulate) {
        ServerFilteringBehaviour filter = BlockEntityBehaviour.get(worldIn, pos, ServerFilteringBehaviour.TYPE);
        InvManipulationBehaviour inserter = BlockEntityBehaviour.get(worldIn, pos, InvManipulationBehaviour.TYPE);
        if (inserter == null)
            return toInsert;
        if (filter != null && !filter.test(toInsert))
            return toInsert;
        if (simulate)
            inserter.simulate();
        class_1799 insert = inserter.insert(toInsert);

        if (!simulate && insert.method_7947() != toInsert.method_7947()) {
            class_2586 blockEntity = worldIn.method_8321(pos);
            if (blockEntity instanceof FunnelBlockEntity funnelBlockEntity) {
                funnelBlockEntity.onTransfer(toInsert);
                if (funnelBlockEntity.hasFlap())
                    funnelBlockEntity.flap(true);
            }
        }
        return insert;
    }

    @Override
    public boolean method_9558(class_2680 state, class_4538 world, class_2338 pos) {
        class_2248 block = world.method_8320(pos.method_10093(getFunnelFacing(state).method_10153())).method_26204();
        return !(block instanceof AbstractFunnelBlock);
    }

    public static boolean isFunnel(class_2680 state) {
        return state.method_26204() instanceof AbstractFunnelBlock;
    }

    @Nullable
    public static class_2350 getFunnelFacing(class_2680 state) {
        if (!(state.method_26204() instanceof AbstractFunnelBlock))
            return null;
        return ((AbstractFunnelBlock) state.method_26204()).getFacing(state);
    }

    protected abstract class_2350 getFacing(class_2680 state);

    @Override
    public Class<FunnelBlockEntity> getBlockEntityClass() {
        return FunnelBlockEntity.class;
    }

    public class_2591<? extends FunnelBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.FUNNEL;
    }
}

==> create-fly-6.0.7-1.21.8-19-sources.jar: com/zurrtum/create/content/logistics/funnel/AbstractFunnelBlock.java <==
package com.zurrtum.create.content.logistics.funnel;

import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.content.equipment.wrench.IWrenchable;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import com.zurrtum.create.foundation.blockEntity.behaviour.BlockEntityBehaviour;
import com.zurrtum.create.foundation.blockEntity.behaviour.filtering.ServerFilteringBehaviour;
import com.zurrtum.create.foundation.blockEntity.behaviour.inventory.InvManipulationBehaviour;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2586;
import net.minecraft.class_2591;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_3218;
import net.minecraft.class_3610;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9904;
import org.jetbrains.annotations.Nullable;

public abstract class AbstractFunnelBlock extends class_2248 implements IBE<FunnelBlockEntity>, IWrenchable, ProperWaterloggedBlock, NeighborUpdateListeningBlock {

    public static final class_2746 POWERED = class_2741.field_12484;

    protected AbstractFunnelBlock(class_2251 p_i48377_1_) {
        super(p_i48377_1_);
        method_9590(method_9564().method_11657(POWERED, false).method_11657(WATERLOGGED, false));
    }

    @Override
    public class_2680 method_9605(class_1750 context) {
        return withWater(method_9564().method_11657(POWERED, context.method_8045().method_49803(context.method_8037())), context);
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pDirection,
        class_2338 pNeighborPos,
        class_2680 pNeighborState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        super.method_9515(builder.method_11667(POWERED, WATERLOGGED));
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 level, class_2338 pos, class_2248 sourceBlock, class_2338 fromPos, boolean isMoving) {
        if (level.field_9236)
            return;
        InvManipulationBehaviour behaviour = BlockEntityBehaviour.get(level, pos, InvManipulationBehaviour.TYPE);
        if (behaviour != null)
            behaviour.onNeighborChanged(fromPos);
    }

    @Override
    public void method_9612(
        class_2680 state,
        class_1937 level,
        class_2338 pos,
        class_2248 block,
        @Nullable class_9904 wireOrientation,
        boolean isMoving
    ) {
        if (level.field_9236)
            return;
        if (!level.method_8397().method_8677(pos, this))
            level.method_64310(pos, this, 1);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 worldIn, class_2338 pos, class_5819 r) {
        boolean previouslyPowered = state.method_11654(POWERED);
        if (previouslyPowered != worldIn.method_49803(pos))
            worldIn.method_8652(pos, state.method_28493(POWERED), class_2248.field_31028);
    }

    public static class_1799 tryInsert(class_1937 worldIn, class_2338 pos, class_1799 toInsert, boolean simulate) {
        ServerFilteringBehaviour filter = BlockEntityBehaviour.get(worldIn, pos, ServerFilteringBehaviour.TYPE);
        InvManipulationBehaviour inserter = BlockEntityBehaviour.get(worldIn, pos, InvManipulationBehaviour.TYPE);
        if (inserter == null)
            return toInsert;
        if (filter != null && !filter.test(toInsert))
            return toInsert;
        if (simulate)
            inserter.simulate();
        class_1799 insert = inserter.insert(toInsert);

        if (!simulate && insert.method_7947() != toInsert.method_7947()) {
            class_2586 blockEntity = worldIn.method_8321(pos);
            if (blockEntity instanceof FunnelBlockEntity funnelBlockEntity) {
                funnelBlockEntity.onTransfer(toInsert);
                if (funnelBlockEntity.hasFlap())
                    funnelBlockEntity.flap(true);
            }
        }
        return insert;
    }

    @Override
    public boolean method_9558(class_2680 state, class_4538 world, class_2338 pos) {
        class_2248 block = world.method_8320(pos.method_10093(getFunnelFacing(state).method_10153())).method_26204();
        return !(block instanceof AbstractFunnelBlock);
    }

    public static boolean isFunnel(class_2680 state) {
        return state.method_26204() instanceof AbstractFunnelBlock;
    }

    @Nullable
    public static class_2350 getFunnelFacing(class_2680 state) {
        if (!(state.method_26204() instanceof AbstractFunnelBlock))
            return null;
        return ((AbstractFunnelBlock) state.method_26204()).getFacing(state);
    }

    protected abstract class_2350 getFacing(class_2680 state);

    @Override
    public Class<FunnelBlockEntity> getBlockEntityClass() {
        return FunnelBlockEntity.class;
    }

    public class_2591<? extends FunnelBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.FUNNEL;
    }
}
